# @push.rocks/smartlog-interfaces
interfaces for the smartlog ecosystem

## Install
To install `@push.rocks/smartlog-interfaces`, ensure you have Node.js and npm installed. Then, run the following command in your project directory:

```sh
npm install @push.rocks/smartlog-interfaces --save
```

This command adds `@push.rocks/smartlog-interfaces` to your package's dependencies and downloads it into the `node_modules` folder.

## Usage

`@push.rocks/smartlog-interfaces` provides TypeScript interfaces for structuring log messages within the Smartlog ecosystem. These interfaces standardize how log data is structured and transmitted across different parts of a system, making it easier to collect, query, and analyze logs.

The Smartlog ecosystem is designed to support a versatile range of logging needs, from simple application logs to complex, structured data logging for microservices. To utilize these interfaces in your project, follow the steps below:

### Setting Up

First, ensure you're using TypeScript in your project. Install TypeScript if you haven't already:

```sh
npm install typescript --save-dev
```

Then, import the necessary interfaces from the package in your logging module or any place you intend to produce logs:

```typescript
import {
  ILogPackage,
  ILogDestination,
  TLogLevel,
  TLogType,
  ILogContext,
  TRuntime,
  TEnvironment,
  ILogCorrelation
} from '@push.rocks/smartlog-interfaces';
```

### Creating a Log Message

Here's how to create a log package conforming to the `ILogPackage` interface:

```typescript
const myLogContext: ILogContext = {
  environment: 'production' as TEnvironment,
  runtime: 'node' as TRuntime,
  zone: 'eu-central-1'
};

const myLogCorrelation: ILogCorrelation = {
  id: '123456789',
  type: 'service'
};

const myLogPackage: ILogPackage = {
  timestamp: Date.now(),
  type: 'log' as TLogType,
  context: myLogContext,
  level: 'info' as TLogLevel,
  correlation: myLogCorrelation,
  message: 'This is a sample log message.',
};
```

In this example, `myLogPackage` encapsulates all the required data for a log message, including contextual data about where and when the log was generated.

### Implementing ILogDestination

`ILogDestination` is an interface that defines how log packages are handled. You can implement your own log destination (e.g., a cloud-based logging service, local log file writer) by conforming to this interface.

```typescript
class MyLogService implements ILogDestination {
  async handleLog(logPackage: ILogPackage): Promise<void> {
    // Implement log handling logic here.
    // For demonstration, we'll simply log to the console.
    console.log(JSON.stringify(logPackage));
  }
}

// Usage
const myLogService = new MyLogService();
await myLogService.handleLog(myLogPackage);
```

This basic implementation of `ILogDestination` offers a foundational structure on which you can build more sophisticated log handling mechanisms, like batch processing, filtering, or routing based on log metadata.

### Extending Functionality

The interfaces provided by `@push.rocks/smartlog-interfaces` serve as a starting point. In a complex application or service, you might need to extend these interfaces with additional properties or create new interfaces that extend these base interfaces to include more specific log data relevant to your application.

#### Custom Log Data

For example, if your application needs to log request-specific information, you could extend `ILogPackage`:

```typescript
interface ILogPackageWithRequestData extends ILogPackage<ILogPackageDataRequest> {
  // You can add more fields or override existing ones
}

// Then, when creating a log package:
const myLogPackageWithRequestData: ILogPackageWithRequestData = {
  // Include all required fields from ILogPackage
  // And add your additional request-specific log data in the 'data' field
};
```

### Conclusion

`@push.rocks/smartlog-interfaces` provides the foundation for consistent and structured logging across your applications and services. By adhering to the interfaces defined in the package, you can ensure that your logs are standardized, making them easier to query, analyze, and understand. Whether you're developing a small application or a large, distributed system, these interfaces offer the flexibility and structure needed to implement robust logging practices.

## License and Legal Information

This repository contains open-source code that is licensed under the MIT License. A copy of the MIT License can be found in the [license](license) file within this repository. 

**Please note:** The MIT License does not grant permission to use the trade names, trademarks, service marks, or product names of the project, except as required for reasonable and customary use in describing the origin of the work and reproducing the content of the NOTICE file.

### Trademarks

This project is owned and maintained by Task Venture Capital GmbH. The names and logos associated with Task Venture Capital GmbH and any related products or services are trademarks of Task Venture Capital GmbH and are not included within the scope of the MIT license granted herein. Use of these trademarks must comply with Task Venture Capital GmbH's Trademark Guidelines, and any usage must be approved in writing by Task Venture Capital GmbH.

### Company Information

Task Venture Capital GmbH  
Registered at District court Bremen HRB 35230 HB, Germany

For any legal inquiries or if you require further information, please contact us via email at hello@task.vc.

By using this repository, you acknowledge that you have read this section, agree to comply with its terms, and understand that the licensing of the code does not imply endorsement by Task Venture Capital GmbH of any derivative works.
